#!/usr/bin/python3.7.3
from mecademicpy.robot import Robot
from NetFT import Sensor
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import pyqtSignal, Qt, QThread, QObject
import os, time


class ForceGuidedJog(QObject):
    completed = pyqtSignal()

    def __init__(self, robot, sensor, parent=None):
        QObject.__init__(self, parent=parent)
        self.continue_run = True
        self.sensor = sensor
        self.r = robot
        self.r.SetTrf(0, 0, 0, 0, 0, -45)
        self.r.ResumeMotion()

    def jog(self):
        maxVel = 250
        while self.continue_run:
            cmd = [i * maxVel for i in self.sensor.norm_values]
            self.r.MoveLinVelTrf(*cmd)
            if self.r.GetStatusRobot().error_status:
                self.r.ResetError()
                self.r.ResumeMotion()
            time.sleep(0.001)
        self.completed.emit()

    def stop(self):
        self.continue_run = False


class TrajRecorder(QObject):
    completed = pyqtSignal()

    def __init__(self, parent=None):
        QObject.__init__(self, parent=parent)
        self.continue_run = True
        self.fb = Robot()
        self.fb.Connect(monitor_mode=True)
        self.last_value = self.fb.GetJoints()

        # Remove previous trajectory log
        if os.path.exists("trajectory_record.prg"):
            os.remove("trajectory_record.prg")

    def record(self):
        while self.continue_run:
            self.current_value = self.fb.GetJoints()
            if self.current_value != self.last_value:
                open("trajectory_record.prg", "a").write(
                    "MoveJoints" + str(self.current_value) + "\n"
                )
                self.last_value = self.current_value
                time.sleep(0.001)
        self.completed.emit()

    def stop(self):
        self.continue_run = False

    def write_data(self, data):
        """Method to record the trajectory"""
        with open("trajectory_record.prg", "a") as log:
            log.write(data)


class ForceGuideWidget(QtWidgets.QWidget):
    ForceJogStop = pyqtSignal()

    def __init__(self):
        super().__init__()
        self.setWindowTitle("Meca500")
        self.setWindowIcon(QtGui.QIcon("favicon.ico"))

        self.setMinimumWidth(350)
        self.setMinimumHeight(350)

        self.logo = QtWidgets.QLabel()
        self.logo.setPixmap(QtGui.QPixmap("logo.png"))

        self.robot_on = QtWidgets.QPushButton(self)
        self.robot_on.setIcon(QtGui.QIcon("robot.png"))
        self.robot_on.setMaximumWidth(100)
        self.robot_on.setMaximumHeight(100)
        self.robot_on.setIconSize(QtCore.QSize(75, 75))
        self.robot_on.setStyleSheet(
            "margin: 1px; padding: 7px; background-color:rgba(125,125,125,0); color: black; border-style: solid; border-radius: 3px; border-width: 0.5px; border-color: black;"
        )
        self.robot_on.setCheckable(True)

        self.enable = QtWidgets.QPushButton(self)
        self.enable.setIcon(QtGui.QIcon("data-wave.png"))
        self.enable.setMaximumWidth(100)
        self.enable.setMaximumHeight(100)
        self.enable.setIconSize(QtCore.QSize(90, 90))
        self.enable.setStyleSheet(
            "margin: 1px; padding: 7px; background-color:rgba(125,125,125,0); color: black; border-style: solid; border-radius: 3px; border-width: 0.5px; border-color: black;"
        )
        self.enable.setCheckable(True)

        self.play = QtWidgets.QPushButton()
        self.play.setIcon(QtGui.QIcon("play.png"))
        self.play.setMaximumWidth(100)
        self.play.setMaximumHeight(100)
        self.play.setIconSize(QtCore.QSize(75, 75))
        self.play.setStyleSheet(
            "margin: 1px; padding: 7px; background-color:rgba(125,125,125,0); color: black; border-style: solid; border-radius: 3px; border-width: 0.5px; border-color: black;"
        )

        self.robot_on.clicked.connect(self.on_robot)
        self.enable.clicked.connect(self.on_enable)
        self.play.clicked.connect(self.on_replay)

        self.hbox = QtWidgets.QHBoxLayout()
        self.hbox.addWidget(self.robot_on)
        self.hbox.addWidget(self.enable)
        self.hbox.addWidget(self.play)

        self.vbox = QtWidgets.QVBoxLayout()
        self.vbox.addWidget(self.logo)
        self.vbox.addLayout(self.hbox)
        self.vbox.addSpacing(100)
        self.setLayout(self.vbox)

    def on_robot(self, pressed):
        if pressed:
            self.robot_on.setStyleSheet(
                "margin: 1px; padding: 7px; background-color:rgba(125,125,125,100); color: black; border-style: solid; border-radius: 3px; border-width: 0.5px; border-color: black;"
            )
            self.robot = Robot()
            self.robot.Connect()
            self.robot.ActivateAndHome()
            self.robot.WaitHomed()
            self.robot.ResetError()
            self.robot.ResumeMotion()
            self.sensor = SensorFeedbackThread("192.168.0.101")
            self.sensor.start()

        else:
            if self.robot is not None:
                self.robot_on.setStyleSheet(
                    "margin: 1px; padding: 7px; background-color:rgba(125,125,125,0); color: black; border-style: solid; border-radius: 3px; border-width: 0.5px; border-color: black;"
                )
                self.robot.DeactivateRobot()
                self.robot.Disconnect()
                # self.slave.DeactivateRobot()
                # self.slave.disconnect()

    def on_enable(self, pressed):
        if pressed:
            self.enable.setStyleSheet(
                "margin: 1px; padding: 7px; background-color:rgba(125,125,125,100); color: black; border-style: solid; border-radius: 3px; border-width: 0.5px; border-color: black;"
            )
            self.start_jog_thread()
            self.play.setEnabled(False)
        else:
            self.enable.setStyleSheet(
                "margin: 1px; padding: 7px; background-color:rgba(125,125,125,0); color: black; border-style: solid; border-radius: 3px; border-width: 0.5px; border-color: black;"
            )
            self.stop_jog_thread()
            self.play.setEnabled(True)

    def on_replay(self):
        self.robot.SetJointVel(20)

        if os.path.exists("trajectory_record.prg"):
            with open("trajectory_record.prg", "r") as log:
                self.movements = log.read().split("\n")

                for i in self.movements:
                    if i is not None:
                        self.robot._send(i)

    def start_jog_thread(self):

        self.thread1 = QThread()
        self.force_guide_jog = ForceGuidedJog(self.robot, self.sensor)
        self.ForceJogStop.connect(
            self.force_guide_jog.stop
        )  # connect stop signal to worker stop method
        self.force_guide_jog.moveToThread(self.thread1)

        self.force_guide_jog.completed.connect(
            self.thread1.quit
        )  # connect the workers finished signal to stop thread
        self.force_guide_jog.completed.connect(
            self.force_guide_jog.deleteLater
        )  # connect the workers finished signal to clean up worker
        self.thread1.finished.connect(
            self.thread1.deleteLater
        )  # connect threads finished signal to clean up thread

        self.thread1.started.connect(self.force_guide_jog.jog)
        self.thread1.finished.connect(self.force_guide_jog.stop)
        self.thread1.start()

        self.thread2 = QThread()
        self.traj_record = TrajRecorder()
        self.ForceJogStop.connect(
            self.traj_record.stop
        )  # connect stop signal to worker stop method
        self.traj_record.moveToThread(self.thread2)

        self.traj_record.completed.connect(
            self.thread2.quit
        )  # connect the workers finished signal to stop thread
        self.traj_record.completed.connect(
            self.traj_record.deleteLater
        )  # connect the workers finished signal to clean up worker
        self.thread2.finished.connect(
            self.thread2.deleteLater
        )  # connect threads finished signal to clean up thread

        self.thread2.started.connect(self.traj_record.record)
        self.thread2.finished.connect(self.traj_record.stop)
        self.thread2.start()

    # When stop_btn is clicked this runs. Terminates the worker and the thread.
    def stop_jog_thread(self):
        self.ForceJogStop.emit()  # emit the finished signal on stop


class SensorFeedbackThread(QThread):
    def __init__(self, ip):
        super().__init__()
        self.sensor = Sensor(ip)
        self.sensor.startStreaming()
        self.value = ()
        self.norm_value = ()

    def run(self):
        self.threadRun = True
        while self.threadRun:
            self.value = list(float(x) for x in self.sensor.measurement())
            self.norm_value = self.norm_force_data(self.value)
        return self.value, self.norm_value

    def norm_force_data(self, val):
        norm_val = [
            val[0] / 40,
            val[1] / 40,
            val[2] / 120,
            val[3] / 16,
            val[4] / 16,
            val[5] / 16,
        ]
        for i in range(0, 6):
            if abs(norm_val[i]) < 0.05:
                norm_val[i] = 0

        return tuple((float(x) for x in norm_val))

    def stop(self):
        self.sensor.stopStreaming()


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    gui = ForceGuideWidget()
    gui.show()
    sys.exit(app.exec_())
